@@ -1,21 +1,22 @@
 # Liblouis Python ctypes bindings
 #
-#  Copyright (C) 2009, 2010 James Teh <jamie@jantrid.net>
+# Copyright (C) 2009, 2010 James Teh <jamie@jantrid.net>
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
-# License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
-#
-# You should have received a copy of the GNU Library General Public
-# License along with this library; if not, write to the
-# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
-# Boston MA  02110-1301 USA.
+# This file is part of liblouis.
+# 
+# liblouis is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published
+# by the Free Software Foundation, either version 2.1 of the License, or
+# (at your option) any later version.
+# 
+# liblouis is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public
+# License along with liblouis. If not, see <http://www.gnu.org/licenses/>.
+# 
 
 """Liblouis Python ctypes bindings
 These bindings allow you to use the liblouis braille translator and back-translator library from within Python.
@@ -44,6 +45,10 @@ def _createTablesString(tablesList):
     """Creates a tables string for liblouis calls"""
     return b",".join([x.encode("ASCII") if isinstance(x, str) else bytes(x) for x in tablesList])
 
+def _createTypeformbuf(length, typeform=None):
+    """Creates a typeform buffer for liblouis calls"""
+    return (c_ushort*length)(*typeform) if typeform else (c_ushort*length)()
+
 createStr = None
 if sys.version_info[0] == 2:
     createStr = lambda x: unicode(x)
@@ -65,38 +70,41 @@ try:
 except NameError:
     # Unix/Cygwin
     _loader = cdll
-liblouis = _loader["liblouis.so.10"]
+liblouis = _loader["liblouis.so.14"]
 
 atexit.register(liblouis.lou_free)
 
 liblouis.lou_version.restype = c_char_p
 
+liblouis.lou_charSize.restype = c_int
+
 liblouis.lou_translateString.argtypes = (
     c_char_p, c_wchar_p, POINTER(c_int), c_wchar_p, 
-         POINTER(c_int), POINTER(c_char), POINTER(c_char), c_int)
+         POINTER(c_int), POINTER(c_ushort), POINTER(c_char), c_int)
 
 liblouis.lou_translate.argtypes = (
     c_char_p, c_wchar_p, POINTER(c_int), c_wchar_p, 
-         POINTER(c_int), POINTER(c_char), POINTER(c_char), 
+         POINTER(c_int), POINTER(c_ushort), POINTER(c_char), 
          POINTER(c_int), POINTER(c_int), POINTER(c_int), c_int)
 
 liblouis.lou_backTranslateString.argtypes = (
          c_char_p, c_wchar_p, POINTER(c_int), c_wchar_p,
-         POINTER(c_int), POINTER(c_char), POINTER(c_char), c_int)
+         POINTER(c_int), POINTER(c_ushort), POINTER(c_char), c_int)
 
 liblouis.lou_backTranslate.argtypes = (
          c_char_p, c_wchar_p, POINTER(c_int), c_wchar_p, POINTER(c_int),
-         POINTER(c_char), POINTER(c_char), POINTER(c_int), POINTER(c_int),
+         POINTER(c_ushort), POINTER(c_char), POINTER(c_int), POINTER(c_int),
          POINTER(c_int), c_int)
 
 liblouis.lou_hyphenate.argtypes = (
          c_char_p, c_wchar_p, c_int, POINTER(c_char), c_int)
 
-liblouis.lou_getTable.argtypes = (c_char_p,)
-liblouis.lou_getTable.restype = c_void_p
+liblouis.lou_checkTable.argtypes = (c_char_p,)
 
 liblouis.lou_compileString.argtypes = (c_char_p, c_char_p)
 
+liblouis.lou_getTypeformForEmphClass.argtypes = (c_char_p, c_char_p)
+
 def version():
     """Obtain version information for liblouis.
     @return: The version of liblouis, plus other information, such as
@@ -105,7 +113,14 @@ def version():
     """
     return liblouis.lou_version().decode("ASCII")
 
-def translate(tableList, inbuf, typeform=None,cursorPos=0, mode=0):
+def charSize():
+    """Obtain charSize information for liblouis.
+    @return: The size of the widechar with which liblouis was compiled.
+    @rtype: int
+    """
+    return liblouis.lou_charSize()
+
+def translate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
     """Translate a string of characters, providing position information.
     @param tableList: A list of translation tables.
     @type tableList: list of str
@@ -133,19 +148,19 @@ def translate(tableList, inbuf, typeform=None,cursorPos=0, mode=0):
     outbuf = create_unicode_buffer(outlen.value)
     typeformbuf = None
     if typeform:
-        typeformbuf = create_string_buffer(struct.pack('B'*len(typeform),*typeform), size=outlen.value)
+        typeformbuf = _createTypeformbuf(outlen.value, typeform)
     inPos = (c_int*outlen.value)()
     outPos = (c_int*inlen.value)()
     cursorPos = c_int(cursorPos)
     if not liblouis.lou_translate(tablesString, inbuf, byref(inlen), 
-                                  outbuf, byref(outlen),  typeformbuf, 
+                                  outbuf, byref(outlen), typeformbuf, 
                                   None, outPos, inPos, byref(cursorPos), mode):
         raise RuntimeError("Can't translate: tables %s, inbuf %s, typeform %s, cursorPos %s, mode %s"%(tableList, inbuf, typeform, cursorPos, mode))
     if isinstance(typeform, list):
-        typeform[:] = typeformbuf.value
+        typeform[:] = list(typeformbuf)
     return outbuf.value, inPos[:outlen.value], outPos[:inlen.value], cursorPos.value
 
-def translateString(tableList, inbuf, typeform = None, mode = 0):
+def translateString(tableList, inbuf, typeform=None, mode=0):
     """Translate a string of characters.
     @param tableList: A list of translation tables.
     @type tableList: list of str
@@ -168,13 +183,13 @@ def translateString(tableList, inbuf, typeform = None, mode = 0):
     outbuf = create_unicode_buffer(outlen.value)
     typeformbuf = None
     if typeform:
-        typeformbuf = create_string_buffer(struct.pack('B'*len(typeform),*typeform), size=outlen.value)
+        typeformbuf = _createTypeformbuf(outlen.value, typeform)
     if not liblouis.lou_translateString(tablesString, inbuf, byref(inlen), 
-                                        outbuf, byref(outlen),  typeformbuf, 
+                                        outbuf, byref(outlen), typeformbuf, 
                                         None, mode):
         raise RuntimeError("Can't translate: tables %s, inbuf %s, typeform %s, mode %s"%(tableList, inbuf, typeform, mode))
     if isinstance(typeform, list):
-        typeform[:] = typeformbuf.value
+        typeform[:] = list(typeformbuf)
     return outbuf.value
 
 def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
@@ -204,7 +219,7 @@ def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
     outbuf = create_unicode_buffer(outlen.value)
     typeformbuf = None
     if isinstance(typeform, list):
-        typeformbuf = create_string_buffer(outlen.value)
+        typeformbuf = _createTypeformbuf(outlen.value)
     inPos = (c_int*outlen.value)()
     outPos = (c_int*inlen.value)()
     cursorPos = c_int(cursorPos)
@@ -213,7 +228,7 @@ def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
                     outPos, inPos, byref(cursorPos), mode):
         raise RuntimeError("Can't back translate: tables %s, inbuf %s, typeform %s, cursorPos %d, mode %d"%(tableList, inbuf, typeform, cursorPos, mode))
     if isinstance(typeform, list):
-        typeform[:] = typeformbuf.value
+        typeform[:] = list(typeformbuf)
     return outbuf.value, inPos[:outlen.value], outPos[:inlen.value], cursorPos.value
 
 def backTranslateString(tableList, inbuf, typeform=None, mode=0):
@@ -239,12 +254,12 @@ def backTranslateString(tableList, inbuf, typeform=None, mode=0):
     outbuf = create_unicode_buffer(outlen.value)
     typeformbuf = None
     if isinstance(typeform, list):
-        typeformbuf = create_string_buffer(outlen.value)
+        typeformbuf = _createTypeformbuf(outlen.value)
     if not liblouis.lou_backTranslateString(tablesString, inbuf, byref(inlen), outbuf,
                     byref(outlen), typeformbuf, None, mode):
         raise RuntimeError("Can't back translate: tables %s, inbuf %s, mode %d"%(tableList, inbuf, mode))
     if isinstance(typeform, list):
-        typeform[:] = typeformbuf.value[:outlen.value]
+        typeform[:] = list(typeformbuf)
     return outbuf.value
 
 def hyphenate(tableList, inbuf, mode=0):
@@ -281,11 +296,10 @@ def checkTable(tableList):
     @param tableList: A list of translation tables.
     @type tableList: list of str
     @raise RuntimeError: If compilation failed.
-    @see: lou_getTable in the liblouis documentation
+    @see: lou_checkTable in the liblouis documentation
     """
-    # TODO: Add this function to the C API and use that instead.
     tablesString = _createTablesString(tableList)
-    if not liblouis.lou_getTable(tablesString):
+    if not liblouis.lou_checkTable(tablesString):
         raise RuntimeError("Can't compile: tables %s"%tableList)
 
 def compileString(tableList, inString):
@@ -302,25 +316,49 @@ def compileString(tableList, inString):
     if not liblouis.lou_compileString(tablesString, inString):
         raise RuntimeError("Can't compile entry: tables %s, inString %s"%(tableList, inString))
 
+def getTypeformForEmphClass(tableList, emphClass):
+    """Get the typeform bit for the named emphasis class.
+    @param tableList: A list of translation tables.
+    @type tableList: list of str
+    @param emphClass: An emphasis class name.
+    @type emphClass: str
+    @see: lou_getTypeformForEmphClass in the liblouis documentation
+    """
+    tablesString = _createTablesString(tableList)
+    return liblouis.lou_getTypeformForEmphClass(tablesString, emphClass)
+
 #{ Typeforms
-plain_text = 0
-italic = 1
-underline = 2
-bold = 4
-computer_braille = 8
+plain_text = 0x0000
+italic = 0x0001       # emph_1
+underline = 0x0002    # emph_2
+bold = 0x0004         # emph_3
+emph_4 = 0x0008
+emph_5 = 0x0010
+emph_6 = 0x0020
+emph_7 = 0x0040
+emph_8 = 0x0080
+emph_9 = 0x0100
+emph_10 = 0x0200
+computer_braille = 0x0400
+no_translate = 0x0800
+no_contract = 0x1000
 #}
 
 #{ Translation modes
 noContractions = 1
 compbrlAtCursor = 2
 dotsIO = 4
-comp8Dots = 8
-pass1Only = 16
 compbrlLeftCursor = 32
-otherTrans = 64
-ucBrl = 128
+ucBrl = 64
+noUndefinedDots = 128
+partialTrans = 256
 #}
 
+if charSize() == 2 and sys.maxunicode != 65535:
+    raise Exception("mismatch in Unicode width: liblouis is 2 and python isn't")
+elif charSize() == 4 and sys.maxunicode != 1114111:
+    raise Exception("mismatch in Unicode width: liblouis is 4 and python isn't")
+
 if __name__ == '__main__':
     # Just some common tests.
     print(version())
